
# qMMC3w194 (quasi MMC3 flush cartridge with mapper194 mode)


qMMC3w194 は CPLDを使用したFC互換機向けのMapper4フラッシュカートリッジです。
拡張したレジスタへの設定で CHR-ROM の一部を CHR-RAM として使用できます。  
qMMC3w194 is a Mapper4 flash cartridge for FC compatible machines using CPLD. 
By setting the extended register, part of the CHR-ROM can be used as CHR-RAM.

## ディレクトリ構成/Directories

|ディレクトリ/Directory|内容/Contents|
|---|---|
|anago_script|kazzo/anago用スクリプト<br>script for kazzo/anago.|
|cupl|ファームウェアおよびソースコード<br>Firmware and source code.|
|eagle|プリント基板のCADデータとガーバーデータ<br>Printed circuit board CAD data and Gerber data.|
|fceux|FCEUXに追加するマッパーのソースコード<br>Mapper source code to add to FCEUX.|
|qmmc3w194demo|機能テスト兼デモプログラム、およびソースコード<br>Functional test and demo program, and source code.|

## 仕様/Specifications

### 概要/Summary

| | |
|---|---|
|PRG ROM capacity|512K|
|PRG ROM window|8K + 8K + 16K fixed|
|PRG RAM capacity|8K|
|PRG RAM window|8K|
|CHR capacity|256K|
|CHR window|2K * 2 + 1K * 4|
|Nametable mirroring|H or V, switchable|
|Bus conflicts|No|
|IRQ|Yes|
|Audio|No|

### バンク/Banks

| | |
|---|---|
|CPU $6000-$7FFF|8KB PRG RAM bank|
|CPU $8000-$9FFF (or $C000-$DFFF)|8KB switchable PRG ROM bank|
|CPU $A000-$BFFF|8KB switchable PRG ROM bank|
|CPU $C000-$DFFF (or $8000-$9FFF)|8KB PRG ROM bank, fixed to the second-last bank|
|CPU $E000-$FFFF|8KB PRG ROM bank, fixed to the last bank|
|PPU $0000-$07FF (or $1000-$17FF)|2KB switchable CHR bank|
|PPU $0800-$0FFF (or $1800-$1FFF)|2KB switchable CHR bank|
|PPU $1000-$13FF (or $0000-$03FF)|1KB switchable CHR bank|
|PPU $1400-$17FF (or $0400-$07FF)|1KB switchable CHR bank|
|PPU $1800-$1BFF (or $0800-$0BFF)|1KB switchable CHR bank|
|PPU $1C00-$1FFF (or $0C00-$0FFF)|1KB switchable CHR bank|

### レジスタ/Registers

Mapper4 互換のため、Nesdev wiki の[MMC3](https://www.nesdev.org/wiki/MMC3)のページを参照してください。  
For Mapper4 compatibility, see the [MMC3](https://www.nesdev.org/wiki/MMC3) page on the Nesdev wiki.

CHR-RAMのモード指定を行えるようにミラーリング設定のレジスタを拡張しています。  
The mirroring setting register has been expanded to allow the CHR-RAM mode to be specified.

#### Mirroring ($A000-$BFFE, even)

```
7  bit  0
---- ----
CCxx xxxM
||      |
||      +- Nametable mirroring (0: vertical; 1: horizontal)
++-------- CHR-RAM mode (see table below.)
```

##### CHR-RAM mode

|$A000.D7|$A000.D6|CHR-RAM|CHR-ROM|note|
|---|---|---|---|---|
|0|0|None|256KB<br>(Bank 0-255)|mapper 4 compatible|
|0|1|2KB<br>(Bank 0-1)|254KB<br>(Bank 2-255)|mapper 194 compatible|
|1|0|8KB<br>(Bank 0-7)|248KB<br>(Bank 8-255)| |
|1|1|32KB<br>(Bank 0-31)|224KB<br>(Bank 32-255)| |


## マッパー番号/Mapper number

Mapper4 または Mapper194 を使用します。  
Use Mapper4 or Mapper194.

このカートリッジを正確にシミュレートするマッパーは
正式なマッパー番号が割り当てられていないため、
プライベート使用に予約されている領域の4072を使用しています。  
The mapper that exactly simulates this cartridge has not been 
assigned an official mapper number, so it uses 4072, 
which is in the area reserved for private use.

kazzo/anago用のスクリプトでは、4 または 194 を使用しています。  
In the script for kazzo/anago, 4 or 194 is used.


## FCEUXへのマッパーの追加方法/How to add a mapper to FCEUX

qmmc3w194.cppをFCEUXのソースコードの下記ディレクトリに配置します。  
Place qmmc3w194.cpp in the following directory of the FCEUX source code.

> src/boards

Visual studioでFCEUXのプロジェクトを開きます。  
ソリューションエクスプローラーでboardsのディレクトリを選択した後、 [プロジェクト][既存の項目の追加]メニューを選択します。  
ファイルダイアログから、qmmc3w194.cppを選択して プロジェクトに追加します。  
Open the FCEUX project in Visual studio.  
Select the boards directory in Solution Explorer, then select [Project]-[Add Existing Item] menu.  
From the File dialog, select qmmc3w194.cpp to add it to your project.   

次に、src/ines.cpp ファイルをエディタで開いて、追加するマッパーの初期化関数を登録します。  
ines.cppの452行目付近にines番号とマッパーの初期化関数を紐づける定義があります。  
Then open the src/ines.cpp file in an editor and register the mapper initialization function you want to add.  
There is a definition that associates the ines number with the mapper initialization function near line 452 of ines.cpp.

```
static BMAPPINGLocal bmap[] = {
	{"NROM",				  0, NROM_Init},
	{"MMC1",				  1, Mapper1_Init},
	{"UNROM",				  2, UNROM_Init},
(略)

	{"",					0, NULL}
};
```

このリストに qMMC3w194 のマッパーの定義を記述します。  
Describe the mapper definition of qMMC3w194 in this list. 

```
	{"qMMC3w194",          4072, QMMC3W194_Init},
```

最後に、src/ines.h ファイルをエディタで開いて、マッパーの初期化関数のプロトタイプ宣言を記述します。  
Finally, open the src/ines.h file in an editor and write the prototype declaration for the mapper initialization function.

```
void QMMC3W194_Init(CartInfo *);
```


## WRAMバックアップ用電池 / WRAM backup battery

CR2032 を使用します。電池リテーナーの形状を理解した上で装着してください。  
裏表を間違えると電極がショートして大変危険です。  
Uses CR2032. Be sure to understand the shape of the battery retainer before installing it.  
If you put it upside down, the electrodes may short out, which is extremely dangerous.


