import argparse
import struct
import os

def parseArg():
	parser = argparse.ArgumentParser()
	parser.add_argument('infile', help='input file name')
	parser.add_argument('outfile', help='output file name')
	parser.add_argument('-bg2', action='store_true')
	parser.add_argument('-bg2z', action='store_true')
	parser.add_argument('-sp2', action='store_true')
	parser.add_argument('-a', action='store_true')
	parser.add_argument('-x', type=int, default=0)
	parser.add_argument('-y', type=int, default=0)
	parser.add_argument('-width', type=int, default=0)
	parser.add_argument('-height', type=int, default=0)
	parser.add_argument('-xstep', type=int, default=0)
	parser.add_argument('-ystep', type=int, default=0)
	parser.add_argument('-n', type=int, default=0)
	parser.add_argument('-uni', type=int, default=0)

	params = parser.parse_args()

	if params.bg2:
		params.xstep = 1
		params.ystep = 1

	if params.bg2z:
		params.xstep = 2
		params.ystep = 2

	if params.sp2:
		params.xstep = 1
		params.ystep = 2

	return params

def convert(ifp, ofp, params):
	# ヘッダーの読み込み
	head = ifp.read(54)
	width, height = struct.unpack('<II', head[18:26])
	pitch = (width + 3) & -4;

	xstep = params.xstep * 8
	ystep = params.ystep * 8
	width2  = params.width  if params.width  != 0 else width
	height2 = params.height if params.height != 0 else height

	# ピクセル変換
	dst = bytearray(16)
	done = False
	n = 0
	offset = 54+1024
	offset += pitch * (height-params.y)
	for h in range(0, height2-(ystep-1), ystep):
		offset -= pitch * ystep
		ifp.seek(offset, os.SEEK_SET)
		pixel = ifp.read(pitch * ystep)
		p1 = pitch * ystep + params.x
		for w in range(0, width2-(xstep-1), xstep):
			p2 = p1
			for y in range(0, ystep, 8):
				p3 = p2
				for x in range(0, xstep, 8):
					p4 = p3
					for j in range(8):
						p4 -= pitch
						data = 0
						for i in range(8):
							data |= ((0x00060500 >> (pixel[p4+i] & 3)) & 0x8080) >> i
						dst[j  ] = (data     ) & 255
						dst[j+8] = (data >> 8) & 255
					if params.uni > 0:
						for v in params.dict:
							if v == dst:
								break
						else:
							params.dict.append(dst.copy())
					else:
						ofp.write(dst)
					n += 1
					if (params.n > 0) and (n >= params.n):
						done = True
						break
					p3 += 8
				p2 -= pitch * 8
				if done:
					break
			p1 += xstep
			if done:
				break
		if done:
			break

	if (params.uni > 0):
		while (len(params.dict) < params.uni):
			params.dict.append(b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff')
		for v in params.dict:
			ofp.write(v)

def main():
	# 引数を解析
	params = parseArg()

	dict = []
	if (params.uni > 0) and params.a:
		with open(params.outfile, 'rb') as f:
			chr = f.read(16)
			while (len(chr) > 0):
				dict.append(chr)
				chr = f.read(16)
	params.dict = dict

	with open(params.infile, 'rb') as ifp:
		with open(params.outfile, \
		  'ab' if (params.uni <= 0) and params.a else 'wb') as ofp:
			convert(ifp, ofp, params)

main()
