import argparse
import os

def parseMemorySize(s):
	r = 0
	if len(s) > 0:
		c = s[-1]
		if c == 'k' or c == 'K':
			r = int(s[:-1]) * 1024
		elif c == 'm' or c == 'M':
			r = int(s[:-1]) * 1024 * 1024
		else:
			r = int(s)

	return r

def parseArg():
	parser = argparse.ArgumentParser()
	parser.add_argument('file', help='nes file')
	parser.add_argument('-num', type=int, default=-1)
	parser.add_argument('-prg_ram', type=parseMemorySize, default='-1')
	parser.add_argument('-chr_ram', type=parseMemorySize, default='-1')

	params = parser.parse_args()

	return params

def ShiftCount(n):
	i = 15
	while i > 0:
		if n & (64<<i):
			break
		i -= 1
	return i

def ParseHeader(head, params):
	params.type = (head[7] >> 2) & 3
	if params.type == 0:
		params.prg_rom_size = head[4]
		params.chr_rom_size = head[5]
		if params.prg_ram < 0:
			params.prg_ram_size = head[8] * 8 * 1024
		else:
			params.prg_ram_size = params.prg_ram
		if params.chr_ram < 0:
			params.chr_ram_size = 8*1024 if head[5] == 0 else 0
		else:
			params.chr_ram_size = params.chr_ram
		params.prg_nvr_size = 0
		params.chr_nvr_size = 0
		if params.num < 0:
			params.num = (head[7] & 0xf0) | ((head[6] & 0xf0) >> 4)
		params.sub          = 0
		params.mirroring    = ((head[6]>>2) & 2) | (head[6] & 1)
		params.save_memory  = ((head[6]>>1) & 1)
		params.tv_system    = ((head[10]<<1) & 2) | ((head[10]>>1) & 1)
		params.bus_conflict = ((head[10]>>5) & 1)
	elif params.type == 2:
		params.prg_rom_size = (((head[9]   )&15)<<8)|head[4]
		params.chr_rom_size = (((head[9]>>4)&15)<<8)|head[5]
		if params.prg_ram < 0:
			params.prg_ram_size = 0 if (head[10]&15)==0 else 64 << (head[10]&15)
		else:
			params.prg_ram_size = params.prg_ram
		if params.chr_ram < 0:
			params.chr_ram_size = 0 if (head[11]&15)==0 else 64 << (head[11]&15)
		else:
			params.chr_ram_size = params.chr_ram
		params.prg_nvr_size = 0 if ((head[10]>>4)&15)==0 else 64 << ((head[10]>>4)&15)
		params.chr_nvr_size = 0 if ((head[11]>>4)&15)==0 else 64 << ((head[11]>>4)&15)
		if params.num < 0:
			params.num = ((head[8] & 0x0f) << 8) | (head[7] & 0xf0) | ((head[6] & 0xf0) >> 4)
		params.sub          = ((head[8]>>4) & 15)
		params.mirroring    = ((head[6]>>2) & 2) | (head[6] & 1)
		params.save_memory  = ((head[6]>>1) & 1)
		params.tv_system    = ((head[12]  ) & 3)
		params.bus_conflict = 0

def BuildHeader(head, params):
	params.type = 0 if params.num < 256 else 2

	if params.type == 0:
		head[ 4] = params.prg_rom_size
		head[ 5] = params.chr_rom_size
		head[ 6] = (head[6] & 0x04) | \
		           (((params.num          )&15)<<4) | \
		           (((params.mirroring >>1)& 1)<<3) | \
		           (((params.save_memory  )& 1)>>1) | \
		           (((params.mirroring    )& 1)   )
		head[ 7] = (head[7] & 0x03) | \
		           (((params.num       >>4)&15)<<4)
		head[ 8] = params.prg_ram_size // (8 * 1034)
		head[ 9] = 1 if params.tv_system == 1 else 0
		head[10] = (((params.bus_conflict)&1)<<5) | \
		           (0x10 if params.prg_ram_size == 0 else 0x00) | \
		           (((params.tv_system   )&1)<<1) | \
		           (((params.tv_system>>1)&1)   )
	elif params.type == 2:
		head[ 4] = params.prg_rom_size & 0xff
		head[ 5] = params.chr_rom_size & 0xff
		head[ 6] = (head[6] & 0x04) | \
		           (((params.num          )&15)<<4) | \
		           (((params.mirroring >>1)& 1)<<3) | \
		           (((params.save_memory  )& 1)<<1) | \
		           (((params.mirroring    )& 1)   )
		head[ 7] = (head[7] & 0x03) | \
		           0x08 | \
		           (((params.num       >>4)&15)<<4)
		head[ 8] = (((params.sub   )&15)<<4) | \
		           (((params.num>>8)&15)   )
		head[ 9] = (((params.chr_rom_size>>8)&15)>>4) | \
		           (((params.prg_rom_size>>8)&15)   )
		head[10] = (ShiftCount(params.prg_nvr_size)<<4) | \
		           (ShiftCount(params.prg_ram_size)   )
		head[11] = (ShiftCount(params.chr_nvr_size)<<4) | \
		           (ShiftCount(params.chr_ram_size)   )
		head[12] = params.tv_system & 3

def main():
	# 引数を解析
	params = parseArg()

	with open(params.file, 'r+b') as fp:
		fp.seek(0, os.SEEK_SET)
		head = bytearray(fp.read(16))

		ParseHeader(head, params)
		BuildHeader(head, params)

		fp.seek(0, os.SEEK_SET)
		fp.write(head)

main()
